"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const fs = require('fs-extra-promise');
const path = require('path');
const util_1 = require('./util');
let projectConfig = {};
global['globalConfig'] = {};
let configValidator = {};
function getConfig(key) {
    return __awaiter(this, void 0, void 0, function* () {
        let result;
        if (typeof key === "string") {
            if (projectConfig[key]) {
                result = projectConfig[key];
            }
            if (globalConfig[key]) {
                result = globalConfig[key];
            }
            if (!result) {
                throw new util_1.CliException(1000, key);
            }
            if (configValidator[key]) {
                var validator = configValidator[key];
                validator(result);
            }
            if (exports.KEY.ANDROID_SUPPORT === key) {
                if (!testAndroidSupportFilePath(result)) {
                    throw new util_1.CliException(1001, "cannot find android-support path or path illegal!");
                }
            }
        }
        else {
            var allConfig = yield getAllConfig();
            result = key.getter(allConfig);
            if (key.validator) {
                key.validator(result);
            }
        }
        return result;
    });
}
exports.getConfig = getConfig;
exports.KEY = {
    ANDROID_SUPPORT: "android_support",
    ANDORID_PROJECT_PATH: "android.project.path",
    ANDROID_SDK: "android-sdk",
    ANT_DIR_PATH: "ant-dir-path"
};
function getAppDataPath() {
    var result;
    switch (process.platform) {
        case 'darwin':
            var home = process.env.HOME || ("/Users/" + (process.env.NAME || process.env.LOGNAME));
            if (!home)
                return null;
            result = `${home}/Library/Application Support/`;
            break;
        case 'win32':
            result = process.env.AppData || `${process.env.USERPROFILE}/AppData/Roaming/`;
            break;
        default:
            ;
    }
    return result;
}
function initConfig(projectPath) {
    return __awaiter(this, void 0, Promise, function* () {
        exports.egretProjectPath = projectPath;
        initProjectConfig();
        yield initGlobalConfig();
    });
}
exports.initConfig = initConfig;
function getAllConfig() {
    return __awaiter(this, void 0, Promise, function* () {
        return Object.assign({}, projectConfig, globalConfig);
    });
}
exports.getAllConfig = getAllConfig;
function initProjectConfig() {
    if (exports.egretProjectPath) {
        let projectConfigFilename = path.join(exports.egretProjectPath, "egret.config");
        if (!fs.existsSync(projectConfigFilename)) {
            projectConfig = {};
        }
        else {
            let content = fs.readFileSync(projectConfigFilename, "utf-8");
            projectConfig = JSON.parse(content);
        }
    }
}
function initGlobalConfig() {
    return __awaiter(this, void 0, void 0, function* () {
        initConfigValidator(exports.KEY.ANDROID_SUPPORT, androidSupportValidator);
        var globalConfigFilename = path.join(getAppDataPath(), "egret.config");
        let isExist = yield fs.existsAsync(globalConfigFilename);
        if (!isExist) {
            globalConfig = {};
        }
        else {
            let content = yield fs.readFileAsync(globalConfigFilename, "utf-8");
            globalConfig = JSON.parse(content);
        }
    });
}
function initConfigValidator(key, validator) {
    configValidator[key] = validator;
}
let androidSupportValidator = (value) => {
    var result = fs.existsSync(value);
    //todo
    // if (false){
    // throw new CliException(1001);
    // }
    return result;
};
function validateConfigs(config, keys) {
    for (var key of keys) {
        var value = getConfig(key);
    }
    return config;
}
exports.validateConfigs = validateConfigs;
function setProjectConfig(key, value) {
    return __awaiter(this, void 0, void 0, function* () {
        console.log("=============", key);
        if (typeof key === 'string') {
            projectConfig[key] = value;
        }
        else {
            key.setter(projectConfig, value);
        }
        var projectConfigFilename = path.join(exports.egretProjectPath, "egret.config");
        var content = JSON.stringify(projectConfig, null, "\t");
        fs.writeFileAsync(projectConfigFilename, content, "utf-8");
    });
}
exports.setProjectConfig = setProjectConfig;
function testAndroidSupportFilePath(value) {
    if (fs.existsSync(value)) {
        let android_support_path = path.join(value, "proj.android");
        if (fs.existsSync(android_support_path)) {
            let propertyFilePath = path.join(android_support_path, "project.properties");
            let manifestFilePath = path.join(android_support_path, "AndroidManifest.xml");
            if (fs.existsSync(propertyFilePath) &&
                fs.existsSync(manifestFilePath)) {
                return true;
            }
        }
    }
    return false;
}
function setGlobaltConfig(key, value) {
    return __awaiter(this, void 0, void 0, function* () {
        if (exports.KEY.ANDROID_SUPPORT === key) {
            if (!testAndroidSupportFilePath(value)) {
                throw new util_1.CliException(1001, "cannot find android-support path or path illegal!");
            }
        }
        globalConfig[key] = value;
        var globalConfigFilename = path.join(getAppDataPath(), "egret.config");
        var content = JSON.stringify(globalConfig, null, "\t");
        fs.writeFileAsync(globalConfigFilename, content, "utf-8");
    });
}
exports.setGlobaltConfig = setGlobaltConfig;
var Selector;
(function (Selector) {
    Selector.android_support_path = {
        getter: c => c.androidProjectPath,
        setter: (c, value) => c.androidProjectPath = value
    };
    Selector.channel_sdk = {
        getter: c => c.sdk,
        setter: (c, value) => {
            c.sdk = value;
        }
    };
})(Selector = exports.Selector || (exports.Selector = {}));
//# sourceMappingURL=config.js.map