"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const xml2js = require('xml2js');
const fs = require('fs-extra-promise');
const path = require('path');
const utils = require('egret-node-utils');
function changeInheritActivity(filePath, libPackageName) {
    return __awaiter(this, void 0, void 0, function* () {
        var fileContent = yield fs.readFileAsync(filePath, "utf-8");
        fileContent = fileContent.replace(/com.egret.androidsupport.GameActivity/, `${libPackageName}.GameActivity`);
        yield fs.writeFileAsync(filePath, fileContent, "utf-8");
    });
}
exports.changeInheritActivity = changeInheritActivity;
function modifyAndroidSDKProject(sdkName, sdkProjectPath, sdkPackageName) {
    return __awaiter(this, void 0, void 0, function* () {
        let prevPackagePath = `src/com/egret/androidsupport${path.sep}${sdkName}`;
        let replaceRex = new RegExp(`com.egret.androidsupport.${sdkName}`, "gi");
        changeProjectPackageName(sdkProjectPath, prevPackagePath, sdkPackageName, replaceRex);
    });
}
exports.modifyAndroidSDKProject = modifyAndroidSDKProject;
function modifyAndroidProject(projectPath, packageName) {
    return __awaiter(this, void 0, void 0, function* () {
        let prevPackagePath = "src/com/egret/androidsupport";
        let replaceRex = /com.egret.androidsupport/gi;
        changeProjectPackageName(projectPath, prevPackagePath, packageName, replaceRex);
    });
}
exports.modifyAndroidProject = modifyAndroidProject;
function addAndroidProjectLib(hostProjectPath, addLibPath, index) {
    return __awaiter(this, void 0, void 0, function* () {
        let propertiesFilePath = path.join(hostProjectPath, "project.properties");
        var fileContent = yield fs.readFileAsync(propertiesFilePath, "utf-8");
        fileContent += `\nandroid.library.reference.${index}=${path.relative(hostProjectPath, addLibPath)}`;
        yield fs.writeFileAsync(propertiesFilePath, fileContent, "utf-8");
    });
}
exports.addAndroidProjectLib = addAndroidProjectLib;
function markAndroidProjectAsLib(projectPath) {
    return __awaiter(this, void 0, void 0, function* () {
        let propertiesFilePath = path.join(projectPath, "project.properties");
        var fileContent = yield fs.readFileAsync(propertiesFilePath, "utf-8");
        fileContent += '\nandroid.library=true';
        yield fs.writeFileAsync(propertiesFilePath, fileContent, "utf-8");
    });
}
exports.markAndroidProjectAsLib = markAndroidProjectAsLib;
function changeProjectPackageName(projectPath, prevPackagePath, packageName, replaceRex) {
    return __awaiter(this, void 0, void 0, function* () {
        let manifestPath = path.join(projectPath, "AndroidManifest.xml");
        let content = yield fs.readFileAsync(manifestPath, "utf-8");
        let obj = yield parseXML(content);
        yield modifyPackageName(projectPath, obj, packageName, prevPackagePath, replaceRex);
        content = yield stringifyXML(obj);
        content = content.replace(replaceRex, packageName);
        yield fs.writeFileAsync(manifestPath, content, "utf-8");
        yield modifyProjectConfig(projectPath);
    });
}
let stringifyXML = (obj) => __awaiter(this, void 0, void 0, function* () {
    var builder = new xml2js.Builder();
    var xml = builder.buildObject(obj);
    return xml;
});
let parseXML = (content) => __awaiter(this, void 0, void 0, function* () {
    return new Promise((reslove, reject) => {
        xml2js.parseString(content, function (err, result) {
            reslove(result);
        });
    });
});
let modifyFile = (path, packageName, replaceRex) => __awaiter(this, void 0, void 0, function* () {
    var fileContent = yield fs.readFileAsync(path, "utf-8");
    fileContent = fileContent.replace(replaceRex, packageName);
    yield fs.writeFileAsync(path, fileContent, "utf-8");
});
function modifyProjectConfig(projectPath) {
    return __awaiter(this, void 0, void 0, function* () {
        let profileConfigPath = path.join(projectPath, ".project");
        let content = yield fs.readFileAsync(profileConfigPath, "utf-8");
        let obj = yield parseXML(content);
        obj.projectDescription.name = path.basename(projectPath, "");
        content = yield stringifyXML(obj);
        yield fs.writeFileAsync(profileConfigPath, content, "utf-8");
    });
}
let modifyPackageName = (projectPath, obj, packageName, prevPackagePath, replaceRex) => __awaiter(this, void 0, void 0, function* () {
    let sourceDir = prevPackagePath;
    let targetDir = path.join("src", packageName.split(".").join(path.sep));
    let allFileList = yield utils.walk(path.join(projectPath, "src"), (p) => fs.statSync(p).isFile());
    let javaFileList = allFileList.filter((path) => path.indexOf(".java") >= 0);
    let mapper = (item, index, arrayIndex) => modifyFile(item, packageName, replaceRex);
    yield Promise.all(javaFileList.map(mapper));
    yield fs.moveAsync(path.join(projectPath, sourceDir), path.join(projectPath, targetDir));
    obj.manifest.$.package = packageName;
});
//# sourceMappingURL=android.js.map